import contextlib
import warnings
from typing import Generator, Union

import requests
import urllib3

from clean_links import __version__

USER_AGENT = f"clean-links/{__version__}"
USER_AGENT = (
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 "
    "(KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"
)
HEADERS = {
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
    "accept-encoding": "gzip, deflate, br",
    "accept-language": "en-US,en;q=0.9",
    "sec-fetch-dest": "document",
    "sec-fetch-mode": "navigate",
    "sec-fetch-site": "same-origin",
    "sec-fetch-user": "?1",
    "upgrade-insecure-requests": "1",
    "user-agent": USER_AGENT,
}


@contextlib.contextmanager
def disable_ssl_warnings() -> Generator:
    with warnings.catch_warnings():
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        yield None


def send(
    session: requests.Session,
    prepped: requests.PreparedRequest,
    history: dict,
    verify: bool,
    timeout: float,
) -> Union[requests.PreparedRequest, None]:
    history["requests"].append(prepped)
    try:
        response = session.send(
            prepped, allow_redirects=False, verify=verify, timeout=timeout
        )
    except requests.exceptions.RequestException as exc:
        exc.history = history
        raise
    else:
        history["responses"].append(response)

    return response.next


def request_redirect_chain(
    session: requests.Session,
    url: str,
    verify: bool,
    timeout: float,
    headers: dict,
    method: str = "HEAD",
) -> dict:
    history: dict = {
        "requests": [],
        "responses": [],
    }

    # prepare initial request
    request = requests.Request(method, url, headers=headers)
    prepped = session.prepare_request(request)

    # send and follow the redirect chain, filling in the history
    next_prepped = send(session, prepped, history, verify, timeout)
    while next_prepped:
        next_prepped = send(session, next_prepped, history, verify, timeout)

    return history


def format_exception(exc: Union[Exception, None]) -> Union[str, None]:
    if exc is None:
        return None
    else:
        return f"{type(exc).__name__}: {exc}"


def unshorten_url(
    url: str, timeout: float = 9, verify: bool = False, headers: dict = HEADERS
) -> dict:
    with requests.Session() as session, disable_ssl_warnings():
        exception = None
        try:
            history = request_redirect_chain(
                session, url, verify, timeout, headers, "HEAD"
            )
        except requests.exceptions.RequestException as exc:
            exception = exc
            history = getattr(exc, "history", {})
            if not history or not history["responses"]:
                raise

        response = history["responses"][-1]
        return {
            "url": url,
            "resolved": response.url,
            "status": response.status_code,
            "exception": format_exception(exception),
            "request_history": [r.url for r in history["requests"]],
            "response_history": [r.status_code for r in history["responses"]],
        }


def main() -> None:
    url = "https://trib.al/5m7fAg3"
    url = "https://www.bloomberg.com/news/articles/2024-01-24/cryptocurrency-ai-electricity-demand-seen-doubling-in-three-years?cmpid%3D=socialflow-twitter-tech&utm_content=tech&utm_medium=social&utm_campaign=socialflow-organic&utm_source=twitter"
    url = "https://tinyurl.com/yc2ft9m5"
    url = "https://bit.ly/3C4WXQ9"
    # url = 'https://tinyurl.com/NewwAlemAndKibrom'
    url = "https://hubs.la/Q01HRjhm0"
    url = "https://expired.badssl.com/"
    url = "https://tinyurl.com/NewwAlemAndKibrom"

    print(unshorten_url(url, verify=True))


if __name__ == "__main__":
    main()



import json


def read_config(
    additional_config_filename: str = "clean_links/clearurls_config.json"
) -> dict:
    with open("clean_links/clearurls.json") as infile:
        clear_urls_rules = dict(json.load(infile))

    with open(additional_config_filename) as infile:
        additional_rules = json.load(infile)

    for provider_name, rules in additional_rules["providers"].items():
        provider = clear_urls_rules["providers"].get(provider_name)
        if not provider:
            clear_urls_rules["providers"][provider_name] = {
                "urlPattern": None,
                "completeProvider": False,
                "rules": [],
                "referralMarketing": [],
                "exceptions": [],
                "rawRules": [],
                "redirections": [],
                "forceRedirection": False,
            }
            provider = clear_urls_rules["providers"][provider_name]

        for key, value in rules.items():
            if key in ["rules"]:
                provider[key].extend(value)
            else:
                provider[key] = value

    return clear_urls_rules



import logging
import re
from urllib.parse import parse_qs, urlencode, urlsplit

from clean_links.config import read_config
from clean_links.unshorten import unshorten_url

clear_urls_rules = read_config()


def query_string(url: str, rules: list) -> str:
    split = urlsplit(url)
    params = parse_qs(split.query)

    delete_keys = {""}
    for rule in rules:
        for key in params:
            if re.match("^" + rule + "$", key, flags=re.IGNORECASE):
                delete_keys.add(key)

    for delete_key in delete_keys:
        params.pop(delete_key, None)

    params_string = urlencode(params, doseq=True)

    if params_string:
        return split.path + "?" + params_string
    else:
        return split.path


def match_provider(provider: str, url: str, rules: dict) -> bool:
    match_url = re.match(rules["urlPattern"], url)
    match_exception = None
    for exception_pattern in rules["exceptions"]:
        try:
            match_exception = re.match(exception_pattern, url)
        except Exception:
            logging.exception(
                f"something's wrong with regex {exception_pattern!r} "
                f"for provider {provider!r}."
            )

        if match_exception:
            break
    return bool(match_url and not match_exception)


def clean_url(
    url: str, keep_query: bool = True, keep_fragment: bool = True
) -> str:
    for provider_name, rules in clear_urls_rules["providers"].items():
        if match_provider(provider_name, url, rules):
            for rule in rules["rawRules"]:
                url = re.sub(rule, "", url, flags=re.IGNORECASE)

            split = urlsplit(url)
            if keep_query:
                full_path = query_string(url, rules["rules"])
            else:
                full_path = split.path

            relative = full_path
            if keep_fragment:
                fragment_path = query_string(split.fragment, rules["rules"])
                if fragment_path:
                    relative += "#" + fragment_path

            url = f"{split.scheme}://{split.netloc}{relative}"

    return url


def main() -> None:
    url = "https://www.amazon.com/Kobo-Glare-Free-Touchscreen-ComfortLight-Adjustable/dp/B0BCXLQNCC/ref=pd_ci_mcx_mh_mcx_views_0?pd_rd_w=Dx5dF&content-id=amzn1.sym.225b4624-972d-4629-9040-f1bf9923dd95%3Aamzn1.symc.40e6a10e-cbc4-4fa5-81e3-4435ff64d03b&pf_rd_p=225b4624-972d-4629-9040-f1bf9923dd95&pf_rd_r=A7JSDJGYR33BN5GRCV7V&pd_rd_wg=xW6Yf&pd_rd_r=4b8a3532-9e28-4857-a929-5e572d2c765f&pd_rd_i=B0BCXLQNCC"

    url = "https://trib.al/5m7fAg3"
    # url = "https://tinyurl.com/yc2ft9m5"
    # url = "https://bit.ly/3C4WXQ9"
    url = "https://tinyurl.com/NewwAlemAndKibrom"
    # url = "https://hubs.la/Q01HRjhm0"
    # url = "https://buff.ly/3Omwkwd"
    # url = "https://bit.ly/48RtRlw"
    # url = "https://srv.buysellads.com/ads/long/x/TCHU7KSHTTTTTTH6NPRNPTTTTTTFNZMBKWTTTTTTA4RZC7VTTTTTTBZI5HINWLB6G3DIEMS4PABU5AIEQQY6BADG2HUT"
    # url = "https://buff.ly/2RjYjMt"

    print(url)
    print()
    resolved = unshorten_url(url).get("resolved", "")
    print(resolved)
    print()
    clear = clean_url(resolved)  # , keep_query=False, keep_fragment=False)
    print(clear)
    # print(url)
    # original, resolved, status = resolve_url(url, 10)
    # print(original)
    # print(resolved)


if __name__ == "__main__":
    main()
